#include <Wire.h>
#include <Adafruit_VL53L0X.h>
#include <Adafruit_NeoPixel.h>

#define LED_PIN    8    // LED燈環的數位引腳設定為8
#define LED_COUNT  16   // 定義LED燈環上的LED數量
#define RED_FLASH_INTERVAL 30000  // 閃紅燈的間隔時間（30秒 = 30000毫秒）
#define STATE_CHANGE_DELAY 300  // 連續0.3秒(300毫秒)狀態不變時才改變燈

Adafruit_VL53L0X lox = Adafruit_VL53L0X();
Adafruit_NeoPixel strip = Adafruit_NeoPixel(LED_COUNT, LED_PIN, NEO_GRB + NEO_KHZ800);

bool milkIsOnBorad = true;  // 初始設為 true 來測試效果
bool previousMilkIsOnBorad = true;  // 上一次的狀態
bool isFlashingRed = false; // 是否正在閃紅燈的標誌
unsigned long previousMillis = 0;  // 記錄上次閃紅燈的時間
unsigned long rainbowMillis = 0;   // 記錄上次彩虹效果更新的時間
unsigned long flashMillis = 0;     // 記錄上次閃紅燈狀態變化的時間
unsigned long stateChangeMillis = 0; // 記錄狀態改變的時間
int rainbowDelay = 20;  // 彩虹變換的速度（延遲時間）
int flashStep = 0;      // 閃紅燈的步驟計數器

void setup() {
  Serial.begin(115200);    // 初始化串口通信
  while (!Serial) { delay(10); }  // 等待串口連接

  strip.begin();
  strip.show();
  strip.setBrightness(50);

  if (!lox.begin()) {
    Serial.println("傳感器初始化失敗");
    while (1);  // 如果傳感器初始化失敗，停留在這裡
  }
}

void loop() {
  VL53L0X_RangingMeasurementData_t measure;
  lox.rangingTest(&measure, false);  // 執行測距

  if (measure.RangeStatus != 4) {  // 4 = 測量失敗
    Serial.print("距離: ");
    Serial.print(measure.RangeMilliMeter);
    Serial.println(" 毫米");
  } else {
    Serial.println("測量失敗");
  }

  unsigned long currentMillis = millis();

  // 根據測得的距離設定 milkIsOnBorad 的狀態
  bool currentMilkIsOnBorad = (measure.RangeMilliMeter >= 50 && measure.RangeMilliMeter <= 130);

  // 檢查狀態是否連續0.3秒不變
  if (currentMilkIsOnBorad != previousMilkIsOnBorad) {
    stateChangeMillis = currentMillis;  // 如果狀態改變，重置計時器
  } else if (currentMillis - stateChangeMillis >= STATE_CHANGE_DELAY) {
    milkIsOnBorad = currentMilkIsOnBorad;  // 狀態連續不變0.3秒，更新狀態
  }

  previousMilkIsOnBorad = currentMilkIsOnBorad;  // 更新上一次的狀態

  // 閃紅燈直到 milkIsOnBorad 變為 false
  if (milkIsOnBorad) {
    if (!isFlashingRed && (currentMillis - previousMillis >= RED_FLASH_INTERVAL)) {
      previousMillis = currentMillis;
      isFlashingRed = true;
      flashStep = 0;  // 重置閃爍步驟計數器
    }
    
    if (isFlashingRed) {
      flashRedNonBlocking(currentMillis);  // 非阻塞地閃紅燈
    } else {
      rainbowCycleStep();  // 顯示彩虹效果
    }
  } 
  else {
    isFlashingRed = false;  // 停止閃紅燈
    setRingColor(strip.Color(255, 255, 0));  // 顯示黃燈
  }
}

// 非阻塞的閃紅燈函式
void flashRedNonBlocking(unsigned long currentMillis) {
  if (currentMillis - flashMillis >= 500) {  // 每500毫秒切換一次狀態
    flashMillis = currentMillis;

    if (flashStep % 2 == 0) {
      setRingColor(strip.Color(255, 0, 0));  // 紅色
    } else {
      setRingColor(strip.Color(0, 0, 0));  // 關閉LED
    }
    
    flashStep++;
  }
}

// 設定整個LED燈環的顏色
void setRingColor(uint32_t color) {
  for (int i = 0; i < strip.numPixels(); i++) {
    strip.setPixelColor(i, color);
  }
  strip.show();
}

// 彩虹效果的逐步函式
void rainbowCycleStep() {
  static int j = 0;
  unsigned long currentMillis = millis();
  
  if (currentMillis - rainbowMillis >= rainbowDelay) {
    rainbowMillis = currentMillis;

    for (int i = 0; i < strip.numPixels(); i++) {
      strip.setPixelColor(i, Wheel(((i * 256 / strip.numPixels()) + j) & 255));
    }
    strip.show();
    j = (j + 1) % (256 * 5);  // 讓j循環在0到1280之間
  }
}

// Wheel函式，用於計算彩虹顏色
uint32_t Wheel(byte WheelPos) {
  WheelPos = 255 - WheelPos;
  if (WheelPos < 85) {
    return strip.Color(255 - WheelPos * 3, 0, WheelPos * 3);
  } else if (WheelPos < 170) {
    WheelPos -= 85;
    return strip.Color(0, WheelPos * 3, 255 - WheelPos * 3);
  } else {
    WheelPos -= 170;
    return strip.Color(WheelPos * 3, 255 - WheelPos * 3, 0);
  }
}
